import compression from 'compression';
import * as sapper from '@sapper/server';
import { createConnection, getConnectionOptions } from "typeorm";
import express from "express";
import session from "cookie-session";
import dotenv from "dotenv";
import { DB_ENTITIES } from "@shared/db/entities";

if(process.env.NODE_ENV == "development") {
	console.log(process.cwd());
    dotenv.config({
        path: "../.env"
    });
    dotenv.config({
        path: "../db.env"
    });
    process.env.TYPEORM_HOST = "localhost";
    process.env.TYPEORM_USERNAME = process.env.DB_USERNAME;
    process.env.TYPEORM_PASSWORD = process.env.DB_PASSWORD;
	process.env.TYPEORM_DATABASE = process.env.DB_NAME;
}


const PORT = +(process.env.PORT as string);
async function main() {
	await createConnection({
		...await getConnectionOptions(),
		entities: DB_ENTITIES
	});

	express()
		.use(
			session({
				maxAge: 604800,
				secret: process.env.ADMIN_COOKIE_KEY,
				name: "session"
			}),
			compression({ threshold: 0 }),
			express.static("static"),
			sapper.middleware({
				session: (req, res) => ({
					user: req.session && req.session.user
				})
			})
		)
		.listen(PORT, err => {
			if (err) console.log('error', err);
		});
}

main();

